; NYAN CAT for the DRAGON 64!
; by Ciaran Anscomb <www@6809.org.uk>

; See README for details.

; ------------------------------------------------------

		include	"dragonhw.s"

sample_cycles	equ	89
sample_rate	equ	894886.25 / sample_cycles	; ~= 10055Hz

; this value helps us keep to 12.5FPS while still getting a tidy conversion
; from the MIDI file:
samp_per_frag	equ	264

; ------------------------------------------------------

		org	$8000

		; convenience for the loader...
		jmp	>start

; the note update code and the playback core both fit within one page of
; memory.  keep DP pointed at this page, and everything should stay really
; fast.

; fetch note data and update playback routine.  note that this contains
; padding so that it always runs in the same amount of time.

; data for the voice in the two wavetable channels only consists of the first
; byte of the wave address.  this is possible because all the waves' data is
; aligned, and each wave is 256 bytes long.

do_frame

		cmpu	#tune_end	; 5
		bhs	10F		; 3 = 8
		bra	20F		; 3 (a)
10		ldu	#tune_start	; 3 (b)
20

		ldb	,u+		; 6
		bmi	10F		; 3 = 9
		;
		exg	a,b		; 8
		exg	a,b		; 8	- preserving b
		exg	a,b		; 8
		exg	a,b		; 8	- preserving b
		abx			; 3
		nop			; 2
		bra	20F		; 3 = 40 (a)
		;
10		pulu	x,y		; 9
		stx	wave1ptr	; 5
		sty	wave1end	; 6
		pulu	x,y		; 9
		stx	wave1restart	; 5
		sty	wave1freq	; 6 = 40 (b)
		;
20		lslb			; 2
		bmi	10F		; 3 = 5
		;
		tfr	b,a		; 6
		tfr	b,a		; 6
		nop			; 2
		bra	20F		; 3 = 17 (a)
		;
10		pulu	a,x		; 8
		sta	wave2ptr	; 4
		stx	wave2freq	; 5 = 17 (b)
		;
20		lslb			; 2
		bmi	10F		; 3 = 5
		;
		tfr	b,a		; 6
		tfr	b,a		; 6
		nop			; 2
		bra	20F		; 3 = 17 (a)
		;
10		pulu	a,x		; 8
		sta	wave3ptr	; 4
		stx	wave3freq	; 5 = 17 (b)
		;
20		; 104 cycles total

; - - - - - - - - - - - - - - - - - - - - - - - - - - -

; the core playback routine.  plays samp_per_frag samples in a tight loop.
; this is based on the core routine from remz' "cocosid", modified to be
; only 3-channel, but with one as sample playback.

play_frag
		pshs	u

		sts	mixer_stmp

; Hack - this value is modified on the fly to gain/lose time over the
; run of animation frames.
mod_count	equ	*+2
		ldd	#samp_per_frag
		std	sample_count

		lds	#reg_pia1_pdra
wave1ptr	equ	*+1
		ldx	#silence_start
wave2ptr	equ	*+1
		ldu	#silence
wave3ptr	equ	*+2
		ldy	#silence

mixer_loop

wave1off	equ	*+2
		ldd	#$0000		; 3
wave1freq	equ	*+1
		addd	#$0000		; 4
		stb	wave1off	; 4
		ldb	,x		; 4
		stb	wave1value	; 4
		leax	a,x		; 5
wave1end	equ	*+1
		cmpx	#$0000		; 4
		bhs	1F		; 3
		bra	2F			; 3 (a)
wave1restart	equ	*+1
1		ldx	#silence_restart	; 3 (b)
2
					; == 34

wave2off	equ	*+1
		ldd	#$0000		; 3
wave2freq	equ	*+1
		addd	#$0000		; 4
		std	wave2off	; 5
		ldb	a,u		; 5
		stb	wave2value	; 4
					; == 21

wave3off	equ	*+1
		ldd	#$0000		; 3
wave3freq	equ	*+1
		addd	#$0000		; 4
		std	wave3off	; 5
		ldb	a,y		; 5
					; == 17

wave1value	equ	*+1
		addb	#$00		; 2
wave2value	equ	*+1
		addb	#$00		; 2
		stb	,s		; 4
					; == 8

		dec	sample_count1	; 6
		bne	mixer_loop	; 3
					; == 9 ... == 89 (mixer loop)

		dec	sample_count0	; 6
		bpl	mixer_loop	; 3
					; == 9 ... == 98 (1 in 256 samples)

		stx	wave1ptr
		lds	mixer_stmp
		puls	u,pc

; - - - - - - - - - - - - - - - - - - - - - - - - - - -

mixer_stmp	rmb	2
decompress_tmp
sample_count
sample_count0	rmb	1
sample_count1	rmb	1

; ------------------------------------------------------

start

		orcc	#$50

		lda	#$80
		tfr	a,dp
		setdp	$80

		sta	reg_sam_tys	; 64K mode
		lds	#$fe00		; account for coco 3

		lda	#$fc
		clr	reg_pia1_cra	; ddr...
		sta	reg_pia1_ddra	; only DAC bits are outputs
		ldd	#$353f
		sta	reg_pia0_crb	; FS enabled hi->lo
		stb	reg_pia1_crb	; CART FIRQ enabled lo->hi
		deca
		sta	reg_pia0_cra	; HS disabled
		sta	reg_pia1_cra	; printer FIRQ disabled

		jsr	decompress_frames

		jsr	generate_misc
		ldu	#intro_start

1		jsr	do_frame
		cmpu	#tune_start
		blo	1B

; timing note: it's important that everything is predictable whatever the tune
; input, but don't need to be precise per-call, so long as any drift is
; corrected within the main loop.

		; 96 lines, 32 bytes per line
		;sta	reg_sam_v0c
		;sta	reg_sam_v1c
		sta	reg_sam_v2s

		; coco3 hack: enable Semigraphics 12, slow down CPU!
		lda	#9
		sta	$ff9c
		sta	reg_sam_r1c
		sta	reg_sam_r0c

		lda	reg_pia0_pdrb	; clear outstanding IRQ
		sync

; - - - - - - - - - - - - - - - - - - - - - - - - - - -

		; Animation runs at 12.5FPS.  This loops is very carefully
		; timed to keep it close to the PAL frame sync.  Over the 12
		; frame run, no time is lost or gained relative to FS.

		; Sadly on 60Hz machines this *will* occasionally glitch
		; (and the time between frames will not be consistent).

loop

		; frame 1
		sta	reg_sam_f0c
		sta	reg_sam_f1s
		sta	reg_sam_f3c
		sta	reg_sam_f4c
		sta	reg_sam_f5c

		jsr	do_frame
		jsr	do_frame
		inc	mod_count	; timing
		jsr	do_frame

		; frame 2
		sta	reg_sam_f0s
		sta	reg_sam_f2s
		sta	reg_sam_f3c	; timing
		sta	reg_sam_f4c	; timing
		nop			; timing

		jsr	do_frame
		jsr	do_frame
		dec	mod_count	; timing
		jsr	do_frame

		; frame 3
		sta	reg_sam_f0c
		sta	reg_sam_f1c
		sta	reg_sam_f3s
		sta	reg_sam_f4c	; timing
		sta	reg_sam_f5c	; timing

		jsr	do_frame
		jsr	do_frame
		jsr	do_frame

		; frame 4
		sta	reg_sam_f0s
		sta	reg_sam_f2c
		sta	reg_sam_f3c
		sta	reg_sam_f4s

		jsr	do_frame
		jsr	do_frame
		inc	mod_count	; timing
		jsr	do_frame

		; frame 5
		sta	reg_sam_f0c
		sta	reg_sam_f1s
		sta	reg_sam_f2s

		jsr	do_frame
		dec	mod_count	; timing
		jsr	do_frame
		jsr	do_frame

		; frame 6
		sta	reg_sam_f0s
		sta	reg_sam_f2c
		sta	reg_sam_f3s

		jsr	do_frame
		jsr	do_frame
		jsr	do_frame

		; frame 7
		sta	reg_sam_f0c
		sta	reg_sam_f1c
		sta	reg_sam_f3c
		sta	reg_sam_f4c
		sta	reg_sam_f5s

		jsr	do_frame
		jsr	do_frame
		inc	mod_count	; timing
		jsr	do_frame

		; frame 8
		sta	reg_sam_f0s
		sta	reg_sam_f2s

		jsr	do_frame
		jsr	do_frame
		jsr	do_frame

		; frame 9
		sta	reg_sam_f0c
		sta	reg_sam_f1s
		sta	reg_sam_f2c
		sta	reg_sam_f3s

		jsr	do_frame
		dec	mod_count	; timing
		jsr	do_frame
		jsr	do_frame

		; frame 10
		sta	reg_sam_f0s
		sta	reg_sam_f2s

		jsr	do_frame
		jsr	do_frame
		jsr	do_frame

		; frame 11
		sta	reg_sam_f0c
		sta	reg_sam_f1c
		sta	reg_sam_f3c
		sta	reg_sam_f4s

		jsr	do_frame
		jsr	do_frame
		jsr	do_frame

		; frame 12
		sta	reg_sam_f0s
		sta	reg_sam_f2c
		sta	reg_sam_f3s

		jsr	do_frame
		jsr	do_frame
		jsr	do_frame

		jmp	loop

; ------------------------------------------------------

generate_misc
		ldx	#silence - 128
1		clr	,x+
		cmpx	#silence + 128
		blo	1B
		ldx	#sqr2 - 128
		ldu	#sqr1 - 128
		bsr	halve_twice
		ldx	#sin2 - 128
		ldu	#sin1 - 128
		bsr	halve_twice
		ldx	#saw2 - 128
		ldu	#saw1 - 128
		bsr	halve_twice
		ldx	#tri2 - 128
		ldu	#tri1 - 128
		bsr	halve_twice
		ldx	#nz02 - 128
		ldu	#nz01 - 128
		bsr	halve_twice
		ldx	#nz12 - 128
		ldu	#nz11 - 128
		bsr	halve_twice
		ldx	#nz22 - 128
		ldu	#nz21 - 128
		bsr	halve_twice
		rts

halve_twice	pshs	u
		bsr	halve_wave
		puls	x
halve_wave	clra
1		ldb	,x+
		asrb
		stb	,u+
		deca
		bne	1B
		rts

; ------------------------------------------------------

; samples before silence, as generally use silence for the "restart"

nya_start
		includebin	"nya_f4.bin"
nya_restart	equ	silence_restart
nya_end

; - - - - - - - - - - - - - - - - - - - - - - - - - - -

silence_start
silence_restart	fcb	$80
silence_end

; ------------------------------------------------------

; midiparse.pl generates tunes that make use of these symbols:

p_channel1	equ	$80
p_channel2	equ	$40
p_channel3	equ	$20

intro_start
		include	"intro.s"
tune_start
		include	"tune.s"
tune_end

; ------------------------------------------------------

; shows a black screen while decompressing the frames
; compressed format is simple:
;    CCCPPNNN (CCC = colour, PP = pixels 0-1, NNN = count)

decompress_frames

		; 16 lines, 32 bytes per line
		sta	reg_sam_v0c
		sta	reg_sam_v1c
		sta	reg_sam_v2c

		ldx	#$0200
		lda	#$80
10		sta	,x+
		cmpx	#$0400
		blo	10B

		lda	reg_pia0_pdrb	; clear outstanding IRQ
		sync

		; base address = $0200
		sta	reg_sam_f0s
		sta	reg_sam_f1c
		sta	reg_sam_f2c
		sta	reg_sam_f3c
		sta	reg_sam_f4c
		sta	reg_sam_f5c

		ldu	#frames
		ldx	#$0400+32

		ldb	#32

100		lda	,u
		anda	#7
		inca
		sta	decompress_tmp
		pulu	a
		lsra
		ora	#$80
		anda	#$fc
		bita	#$08
		beq	10F
		ora	#$02
10		bita	#$04
		beq	20F
		ora	#$01
20		sta	-32,x
		sta	,x+
		decb
		bne	30F
		ldb	#32
		leax	32,x
30		dec	decompress_tmp
		bne	20B

		cmpx	#$7e00
		blo	100B
		
		rts

		fcc	0,"http://www.6809.org.uk/dragon/",0

frames		includebin	"cframes.bin"

; ------------------------------------------------------

sqr2		equ	*+128
		include	"wave_square.s"
sin2		equ	*+128
		include	"wave_sine.s"
saw2		equ	*+128
		include	"wave_saw.s"
tri2		equ	*+128
		include	"wave_triangle.s"
nz02		equ	*+128
		includebin	"nz0.bin"
nz12		equ	*+128
		includebin	"nz1.bin"
nz22		equ	*+128
		includebin	"nz2.bin"

silence		equ	*+128
		rmb	256
sqr1		equ	*+128
		rmb	256
sqr0		equ	*+128
		rmb	256
sin1		equ	*+128
		rmb	256
sin0		equ	*+128
		rmb	256
saw1		equ	*+128
		rmb	256
saw0		equ	*+128
		rmb	256
tri1		equ	*+128
		rmb	256
tri0		equ	*+128
		rmb	256
nz01		equ	*+128
		rmb	256
nz00		equ	*+128
		rmb	256
nz11		equ	*+128
		rmb	256
nz10		equ	*+128
		rmb	256
nz21		equ	*+128
		rmb	256
nz20		equ	*+128
		rmb	256

end		equ	*
