#!/usr/bin/perl

use Math::Trig ':pi';

my $mode = shift @ARGV;

if ($mode eq 'sine') {
	my $scale = shift @ARGV || 42;
	my $clamp = shift @ARGV || $scale;
	for my $i (0..255) {
		my $r = (pi2*$i) / 255;
		my $v = int(sin($r) * $scale);
		if ($v < -$clamp) { $v = -$clamp; }
		if ($v > $clamp) { $v = $clamp; }
		outval($i, $v);
	}
	exit 0;
}

if ($mode eq 'saw') {
	my $scale = shift @ARGV || 42;
	my $clamp = shift @ARGV || $scale;
	for my $i (0..255) {
		my $v = int((($i - 127.5) / 127.5) * $scale);
		if ($v < -$clamp) { $v = -$clamp; }
		if ($v > $clamp) { $v = $clamp; }
		outval($i, $v);
	}
	exit 0;
}

if ($mode eq 'triangle') {
	my $scale = shift @ARGV || 42;
	my $clamp = shift @ARGV || $scale;
	for my $i (0..255) {
		my $v;
		if ($i < 128) {
			$v = int((($i - 63.5) / 63.5) * $scale);
		} else {
			$v = int(((191.5 - $i) / 63.5) * $scale);
		}
		if ($v < -$clamp) { $v = -$clamp; }
		if ($v > $clamp) { $v = $clamp; }
		outval($i, $v);
	}
	exit 0;
}

if ($mode eq 'square') {
	my $scale = shift @ARGV || 42;
	my $offset = shift @ARGV || 0;
	rect($scale, 128, $offset);
	exit 0;
}

if ($mode eq 'rect') {
	my $scale = shift @ARGV || 42;
	my $duty = shift @ARGV || 64;
	my $offset = shift @ARGV || 256-$duty;
	rect($scale, $duty, $offset);
	exit 0;
}

sub rect {
	my ($scale, $duty, $offset) = @_;
	for my $i (0..255) {
		my $v = -$scale;
		if ((($i - $offset) % 256) < $duty) {
			$v = $scale;
		}
		outval($i, $v);
	}
	exit 0;
}

sub outval {
	my ($i, $v) = @_;
	if (($i % 16) == 0) {
		print "\tfcb\t";
	} else {
		print ", ";
	}
	print $v;
	if (($i % 16) == 15) {
		print "\n";
	}
}
