#!/usr/bin/perl

# Compress a stream of semigraphics bytes
#
# Assumes a subdivided screen, and ONLY includes bits 0-1 of pixel data.
#
# Output format is CCCPPNNN (CCC = colour, PP = pixel bits, NNN = count)

use IO::Handle;

sub suggest_help {
	print STDERR "Try '$0 --help' for more information'\n";
	exit 0;
}

sub help_text {
	print STDERR <<EOF;
usage: $0 OPTION... input-file
Compress a stream of SemiGraphics bytes.

  -o FILE              output file (defaults to stdout)
EOF
	exit 0;
}

while (my $opt = shift @ARGV) {
	if ($opt eq '--') {
		last;
	} elsif ($opt eq '-o') {
		my $f = shift @ARGV;
		my $o;
		open($o, ">", $f) or die $!;
		STDOUT->fdopen(\*$o, 'w') or die $!;
	} elsif ($opt eq '--help') {
		help_text();
	} elsif ($opt =~ /^-/) {
		print STDERR "Unrecognised option '$opt'\n";
		suggest_help();
	} else {
		unshift @ARGV, $opt;
		last;
	}
}

die "no input file\n" unless scalar(@ARGV) > 0;
my $file = shift @ARGV;

my $in;
open($in, "<", $file) or die "failed to open $file: $!\n";
binmode $in;
binmode STDOUT;

my $last_byte;
my $b;
while (sysread($in, $b, 1) > 0) {
	$b = unpack("C", $b);
	if (!defined $last_byte) {
		$last_byte = $b;
		$count = 1;
		next;
	}
	if ($b != $last_byte) {
		output_run($last_byte, $count);
		$last_byte = $b;
		$count = 1;
		next;
	}
	$count++;
	if ($count == 8) {
		output_run($last_byte, 8);
		undef $last_byte;
		$count = 0;
	}
}
if ($count > 0) {
	output_run($last_byte, $count);
}

sub output_run {
	my ($ib,$count) = @_;
	$count--;
	my $ob = (($ib << 1) & 0xe0) | (($ib << 3) & 0x18) | $count;
	print pack("C", $ob);
}
